/* Tabela TABEPREC: TABELA DE PREOS*/
/* Descrio: A trigger insere registros na tabela LOGOPERAVND a partir de alteraes na tabela TABEPREC */
/* Objetivo: Gerar um log que vai registrar as alteraes na TABEPREC,
             utilizando a tabela de Logs de Operaes do VND*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'LOG_VND_TABEPREC'   AND
                 TYPE   = 'TR' 	               AND
	              U.UID  = O.UID           AND
	              U.NAME = USER)
DROP TRIGGER LOG_VND_TABEPREC
GO


CREATE TRIGGER LOG_VND_TABEPREC
ON TABEPREC
FOR INSERT, UPDATE, DELETE
AS
  DECLARE @V_DSREMOTEADDR VARCHAR(15);
  DECLARE @V_DSREMOTEHOST VARCHAR(15);
  DECLARE @V_DSSERVERRDM  VARCHAR(20);
  DECLARE @V_DSSERVMETHOD VARCHAR(30);
  DECLARE @V_DSREMOTEOPER VARCHAR(12);
  DECLARE @V_DESC         VARCHAR(500);
  DECLARE @V_ITPESQ       VARCHAR(40);
  DECLARE @V_NMOPERADOR   VARCHAR(40);
  DECLARE @V_IDTIPOOPER   VARCHAR(1);

/*VARIVEIS DO CURSOR C_CURSOR*/

  DECLARE @V_ID_COMANDO    VARCHAR(1);

  DECLARE @V_OLD_CDFILIAL            varchar(4);
  DECLARE @V_OLD_CDTABEPREC          varchar(3);
  DECLARE @V_OLD_NMTABEPREC          varchar(20);
  DECLARE @V_OLD_DTULTATUTBPR        datetime;

  DECLARE @V_NEW_CDFILIAL            varchar(4);
  DECLARE @V_NEW_CDTABEPREC          varchar(3);
  DECLARE @V_NEW_NMTABEPREC          varchar(20);
  DECLARE @V_NEW_DTULTATUTBPR        datetime;


BEGIN
/* IDTIPOOPER - '1' - Incluso, '2' - Alterao, '3' - Excluso*/

   SET @V_DESC   = '';
   SET @V_ITPESQ     = '';
   SET @V_NMOPERADOR = '';
   SET @V_IDTIPOOPER = '';

   EXEC NMUSUARIO @P_NMUSUARIO = @V_NMOPERADOR OUTPUT;
   SET @V_NMOPERADOR = ISNULL(@V_NMOPERADOR, ' ');

   EXEC WEBTUNNING  @P_DSREMOTEADDR = @V_DSREMOTEADDR OUTPUT,
                    @P_DSREMOTEHOST = @V_DSREMOTEHOST OUTPUT,
                    @P_DSSERVERRDM  = @V_DSSERVERRDM OUTPUT,
                    @P_DSSERVMETHOD = @V_DSSERVMETHOD OUTPUT,
                    @P_DSREMOTEOPER = @V_DSREMOTEOPER OUTPUT;

   DECLARE C_CURSOR CURSOR FOR
     SELECT
     'D'   ID_COMANDO,
           CDFILIAL,
           CDTABEPREC,
           NMTABEPREC,
           DTULTATUTBPR,

           NULL,NULL,NULL,NULL

     FROM
           DELETED
     WHERE     
           CDFILIAL+CDTABEPREC NOT IN (SELECT CDFILIAL+CDTABEPREC FROM INSERTED)
     UNION ALL

     SELECT
     'U'   IDCOMANDO,
           DELETED.CDFILIAL,
           DELETED.CDTABEPREC,
           DELETED.NMTABEPREC,
           DELETED.DTULTATUTBPR,

           INSERTED.CDFILIAL,
           INSERTED.CDTABEPREC,
           INSERTED.NMTABEPREC,
           INSERTED.DTULTATUTBPR

     FROM
           INSERTED,
           DELETED
     WHERE     
           INSERTED.CDFILIAL   = DELETED.CDFILIAL AND
           INSERTED.CDTABEPREC = DELETED.CDTABEPREC
     UNION ALL

     SELECT
     'I'   IDCOMANDO,
           NULL,NULL,NULL,NULL,

           INSERTED.CDFILIAL,
           INSERTED.CDTABEPREC,
           INSERTED.NMTABEPREC,
           INSERTED.DTULTATUTBPR

     FROM
           INSERTED
     WHERE     
           CDFILIAL+CDTABEPREC NOT IN (SELECT CDFILIAL+CDTABEPREC FROM DELETED)

     OPEN C_CURSOR;
           FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,

					 @V_OLD_CDFILIAL,@V_OLD_CDTABEPREC,
					 @V_OLD_NMTABEPREC,@V_OLD_DTULTATUTBPR,

					 @V_NEW_CDFILIAL,@V_NEW_CDTABEPREC,
					 @V_NEW_NMTABEPREC,@V_NEW_DTULTATUTBPR

     WHILE (@@FETCH_STATUS = 0) BEGIN
     
       IF @V_ID_COMANDO= 'U' /*UPDATE*/ BEGIN
		IF ISNULL(@V_NEW_CDFILIAL,'NULL') <> ISNULL(@V_OLD_CDFILIAL,'NULL') BEGIN
			SET @V_DESC   =  'Filial ' +  ISNULL(@V_NEW_CDFILIAL,'');
		END;
		IF ISNULL(@V_NEW_CDTABEPREC,'NULL') <> ISNULL(@V_OLD_CDTABEPREC,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + '  Cd da Tabela de Preo ' + ISNULL(@V_NEW_CDTABEPREC,'');
		END;
		IF ISNULL(@V_NEW_NMTABEPREC,'NULL') <> ISNULL(@V_OLD_NMTABEPREC,'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + '  Nm da Tabela de Preo ' + ISNULL(@V_NEW_NMTABEPREC,'');
		END;
		IF ISNULL(CONVERT(VARCHAR,@V_NEW_DTULTATUTBPR),'NULL') <> ISNULL(CONVERT(VARCHAR,@V_OLD_DTULTATUTBPR),'NULL') BEGIN
            SET @V_DESC  = ISNULL(@V_DESC,'') + '  Dt Ult Atu do Preo ' + ISNULL(CONVERT(VARCHAR,@V_NEW_DTULTATUTBPR),'');
		END;
          SET @V_IDTIPOOPER = '2';                                                             
       END;       
       IF @V_ID_COMANDO= 'I' /*INSERO*/ BEGIN
          SET @V_DESC   =  'Filial '                  +  ISNULL(@V_NEW_CDFILIAL,''    )     +
                           ' Cd da Tabela de Preo '  +  ISNULL(@V_NEW_CDTABEPREC,''  )     +
                           ' Nm da Tabela de Preo '  +  ISNULL(@V_NEW_NMTABEPREC,'' )      + 
                           ' Dt Ult Atu do Preo '    +  ISNULL(CONVERT(VARCHAR,@V_NEW_DTULTATUTBPR),'');
          SET @V_IDTIPOOPER = '1';
       END
       IF @V_ID_COMANDO= 'D' /*DELEO*/  BEGIN
          SET @V_DESC   =  'Filial '                  +  ISNULL(@V_OLD_CDFILIAL,''    )     +
                           ' Cd da Tabela de Preo '  +  ISNULL(@V_OLD_CDTABEPREC,''  )     +
                           ' Nm da Tabela de Preo '  +  ISNULL(@V_OLD_NMTABEPREC,'' )      +
                           ' Dt Ult Atu do Preo '    +  ISNULL(CONVERT(VARCHAR,@V_OLD_DTULTATUTBPR),'');
          SET @V_IDTIPOOPER = '3';
       END;
       IF  (@V_IDTIPOOPER <> '') BEGIN
          IF @V_DESC IS NOT NULL BEGIN
             SET @V_ITPESQ =  ISNULL(@V_NEW_CDFILIAL+@V_NEW_CDTABEPREC,@V_OLD_CDFILIAL+@V_OLD_CDTABEPREC);
             SET @V_DESC = SUBSTRING(@V_DESC, 1, 300);
             INSERT INTO LOGOPERAVND
                    (NMTABELAVND, DTOPERACAOVND, NMRESPOPERVND, IDTIPOOPERVND, DSOPERACAOVND,
                     CDPESQITEMVND, DSRDMUSUOPERVND, DSENDUSUOPERVND, DSNMUSUOPERVND, DSMETUSUOPERVND,
                     CDORESPOPERVND, CDFILLOGOPERVND)
             VALUES ('TABEPREC',GETDATE(), ISNULL(@V_NMOPERADOR,' '), @V_IDTIPOOPER, @V_DESC, @V_ITPESQ,
                     @V_DSSERVERRDM, @V_DSREMOTEADDR, @V_DSREMOTEHOST, @V_DSSERVMETHOD, @V_DSREMOTEOPER,
                     ISNULL(@V_NEW_CDFILIAL,@V_OLD_CDFILIAL));
          END;
       END;
       FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,

				     @V_OLD_CDFILIAL,@V_OLD_CDTABEPREC,
   				     @V_OLD_NMTABEPREC,@V_OLD_DTULTATUTBPR,

                                     @V_NEW_CDFILIAL,@V_NEW_CDTABEPREC,
				     @V_NEW_NMTABEPREC,@V_NEW_DTULTATUTBPR

     SET @V_DESC   = '';
     END;
   CLOSE      C_CURSOR;
   DEALLOCATE C_CURSOR;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'LOG_VND_TABEPREC','5.20.001';
GO 